﻿using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Pdf;
using Kivii.Text.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace Kivii.Lims.Addons
{
    [Route(Configs.RouteFileMerge)]
    public class FileMerge
    {
        public List<Guid> Kvids { get; set; }

        public string FolderPath { get; set; } = "/Reports/Pdf";
        public bool Download { get; set; } = true;
    }
    public class FileMergeService:Service
    {
       public object Any(FileMerge request)
        {
            request.Kvids.ThrowIfNullOrEmpty("请传入报告集合以合并PDF");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryPdfEntityFile = conn.From<EntityDbFile<Report>>();
            queryPdfEntityFile.Where(o => o.OwnerKvid != Guid.Empty && o.ParentKvid == Guid.Empty && o.DbFolderPath ==request.FolderPath && o.Extension.ToLower() == ".pdf" && Sql.In(o.OwnerKvid, request.Kvids));
            var pdfEntityFiles = conn.Select(queryPdfEntityFile);
            pdfEntityFiles.ThrowIfNullOrEmpty("未找到指定pdf文件");

            pdfEntityFiles = pdfEntityFiles.OrderBy(o => Array.IndexOf(request.Kvids.ToArray(), o.OwnerKvid)).ToList();

            if (pdfEntityFiles.Count == 1)
            {
                var dbFile = pdfEntityFiles[0];
                var physicalStorageFilePath = dbFile.GetPhysicalPath();
                if (!File.Exists(physicalStorageFilePath)) throw new Exception("服务器端文件已被移除");
                FileInfo localFile = new FileInfo(physicalStorageFilePath);
                return new HttpResult(localFile, asAttachment: request.Download, fileName: dbFile.Name);
            }
            var fileName = $"{DateTime.Now:yyyyMMddHHmmss}合并PDF.pdf";
            var tempPath = KiviiContext.GetTemporaryFilePath(fileName);
            pdfEntityFiles.MergePDF(tempPath);
            FileInfo fileInfo = new FileInfo(tempPath);
            return new HttpResult(fileInfo, asAttachment: request.Download, fileName: fileName);
        }
    }

    public static class FileMergeExtension
    {
        /// <summary>
        /// 合并多个pdf
        /// </summary>
        /// <param name="pdfEntityFiles">pdf文件路径集合</param>
        /// <param name="outPath">最终pdf的输出目录</param>
        public static void MergePDF(this List<EntityDbFile<Report>> pdfEntityFiles, string outPath)
        {
            Document source = new Document();
            PdfCopy pdfCopy =new PdfCopy(source,new FileStream(outPath, FileMode.Create));
            source.Open();
            for (int i = 0; i < pdfEntityFiles.Count; i++)
            {
                var dbFile = pdfEntityFiles[i];
                var physicalStorageFilePath = dbFile.GetPhysicalPath();

                PdfReader reader = new PdfReader(physicalStorageFilePath);

                int iPageNum = reader.NumberOfPages;
                for (int j = 1; j <= iPageNum; j++)
                {
                    PdfImportedPage importedPage = pdfCopy.GetImportedPage(reader, j);
                    pdfCopy.AddPage(importedPage);
                }
                reader.Close();
            }
            source.Close();
        }
    }
    
}
